Imports System.Text
Imports System.Threading.Tasks

Imports Emotiv
Imports Microsoft.Azure.Devices.Client
Imports Newtonsoft.Json
Imports System.Console

Namespace BrainComputerInterface
	Friend Class Program
		Private Shared stopHeadsetInformation As Boolean = True 'GetHeadsetInformation()
		Private Shared userID As Integer = -1 'GetBrainInterfacePower()
		Private Shared deviceClient As DeviceClient
		Private Shared _iotHubUri As String = "ADD YOUR IOT NAME.azure-devices.net"
		Private Shared _deviceKey As String = "ADD DEVICE KEY"
		Private Shared _ActivityId As Integer = 0
		Private Shared _DeviceId As Integer = 1
		Private Shared _UserName As String = "BrainComputerInterface001"

		Shared Sub Main(ByVal args() As String)
			WriteLine("You are uploading to: " & _iotHubUri)
			WriteLine("Current username is:  " & _UserName & "with device id of: " & _DeviceId.ToString)
			WriteLine()
			WriteLine(" 1  - *Baseline")
			WriteLine(" 3  - Writing")
			WriteLine(" 4  - *Reading")
			WriteLine(" 7  - Relaxation")
			WriteLine(" 9  - *Watching media")
			WriteLine(" 10 - Public Speaking")
			WriteLine(" 11 - *Listening to Music")
			WriteLine(" 13 - *Working at Computer")
			WriteLine(" 14 - Programming")
			WriteLine(" 16 - Meeting")
			WriteLine(" 17 - Exercising")
			WriteLine(" 19 - *Meditation")
			WriteLine(" XX - *Playing Music(20)*")
			WriteLine(" 21 - Visualize Push")
			WriteLine(" 22 - Visualize Pull")
			WriteLine()
			Write("Enter what you will be doing while uploading the brain waves: ")

			Try
				_ActivityId = Convert.ToInt32(ReadLine)
				WriteLine("You selected: " & _ActivityId.ToString)

				WriteLine("Hit any key to begin uploading your brain waves")
				ReadLine()
				WriteLine("Session Starting...")

				deviceClient = DeviceClient.Create(_iotHubUri, New DeviceAuthenticationWithRegistrySymmetricKey(_UserName, _deviceKey))
				GetHeadsetInformation()
				GetBrainInterfaceMeasurements()
				ReadLine()
			Catch ex As Exception
				WriteLine("Please make sure you entered the number.")
				WriteLine(ex.Message)
				ReadLine()
			End Try
		End Sub

		Private Shared Sub GetHeadsetInformation()
            Dim engine As EmoEngine = EmoEngine.Instance
            AddHandler engine.EmoStateUpdated, AddressOf engine_EmoStateUpdated
			engine.Connect()

			Do While stopHeadsetInformation
				If KeyAvailable Then
					Exit Do
				End If
				engine.ProcessEvents()
				If CInt(Fix(userID)) = -1 Then
					Continue Do
				End If
			Loop
			engine.Disconnect()
			BackgroundColor = ConsoleColor.Black
			WriteLine("I am completly operational and all systems are go")
			RemoveHandler engine.EmoStateUpdated, AddressOf engine_EmoStateUpdated
		End Sub

		Private Shared Sub engine_EmoStateUpdated(ByVal sender As Object, ByVal e As EmoStateUpdatedEventArgs)
			Dim es As EmoState = e.emoState
			If e.userId <> 0 Then
				Exit Sub
			End If

			Dim timeFromStart As Single = es.GetTimeFromStart
			Dim signalStrength As EdkDll.IEE_SignalStrength_t = es.GetWirelessSignalStatus
			Dim maxChargeLevel As Int32 = 0
			Dim chargeLevel As Int32 = 0
			es.GetBatteryChargeLevel(chargeLevel, maxChargeLevel)

			If chargeLevel > 0 AndAlso signalStrength > EdkDll.IEE_SignalStrength_t.BAD_SIG Then
				stopHeadsetInformation = False
				BackgroundColor = ConsoleColor.DarkGreen
			End If

			WriteLine("Time: " & timeFromStart)
			WriteLine("Signal Strength: " & signalStrength)
			WriteLine("Charge Level: " & chargeLevel.ToString)
			WriteLine("Max Charge Level: " & maxChargeLevel.ToString)

			WriteLine("AF3: " & CInt(Fix(es.GetContactQuality(CInt(Fix(EdkDll.IEE_InputChannels_t.IEE_CHAN_AF3))))))
			WriteLine("T7:  " & CInt(Fix(es.GetContactQuality(CInt(Fix(EdkDll.IEE_InputChannels_t.IEE_CHAN_T7))))))
			WriteLine("Pz : " & CInt(Fix(es.GetContactQuality(CInt(Fix(EdkDll.IEE_InputChannels_t.IEE_CHAN_O1))))))
			WriteLine("T8 : " & CInt(Fix(es.GetContactQuality(CInt(Fix(EdkDll.IEE_InputChannels_t.IEE_CHAN_T8))))))
			WriteLine("AF4: " & CInt(Fix(es.GetContactQuality(CInt(Fix(EdkDll.IEE_InputChannels_t.IEE_CHAN_AF4))))))
		End Sub

		Private Shared Sub GetBrainInterfaceMeasurements()
			Dim engine As EmoEngine = EmoEngine.Instance
			AddHandler engine.UserAdded, AddressOf engine_UserAdded_Event
			engine.Connect()

			Dim channelList() As EdkDll.IEE_DataChannel_t = { EdkDll.IEE_DataChannel_t.IED_AF3, EdkDll.IEE_DataChannel_t.IED_AF4, EdkDll.IEE_DataChannel_t.IED_T7, EdkDll.IEE_DataChannel_t.IED_T8, EdkDll.IEE_DataChannel_t.IED_O1 }

			Do
				engine.ProcessEvents(10)
				If userID < 0 Then
					Continue Do
				End If

				If KeyAvailable Then
					Exit Do
				End If

				Dim alpha(0) As Double
				Dim low_beta(0) As Double
				Dim high_beta(0) As Double
				Dim gamma(0) As Double
				Dim theta(0) As Double

				For i  = 0 To 4
					engine.IEE_GetAverageBandPowers(CUInt(userID), channelList(i), theta, alpha, low_beta, high_beta, gamma)
					If theta(0) > 0 Then
'						#Region "TestOutput"
						'WriteLine("Channel: " + channelListi);
						'WriteLine(
						'      "Theta:     " + theta0.ToString.Substring(0, 5) +
						'      ", Alpha:     " + alpha0.ToString.Substring(0, 5) +
						'      ", Low Beta:  " + low_beta0.ToString.Substring(0, 5) +
						'      ", High Beta: " + high_beta0.ToString.Substring(0, 5) +
						'      ", Gamma:     " + gamma0.ToString.Substring(0, 5));
'						#End Region
						SendBrainMeasurementToAzureAsync(channelList(i).ToString, theta(0).ToString.Substring(0, 7), alpha(0).ToString.Substring(0, 7), low_beta(0).ToString.Substring(0, 7), high_beta(0).ToString.Substring(0, 7), gamma(0).ToString.Substring(0, 7))
					End If
				Next i
			Loop
			engine.Disconnect()
		End Sub

		Private Shared Sub engine_UserAdded_Event(ByVal sender As Object, ByVal e As EmoEngineEventArgs)
			WriteLine("User Added Event has occured")
			userID = CInt(Fix(e.userId))
			WriteLine("UserID = " & userID.ToString)
			EmoEngine.Instance.IEE_FFTSetWindowingType(CUInt(userID), EdkDll.IEE_WindowingTypes.IEE_HAMMING)
		End Sub

		Private Shared async Sub SendBrainMeasurementToAzureAsync(ByVal channel As String, ByVal theta As String, ByVal alpha As String, ByVal lowbeta As String, ByVal highbeta As String, ByVal gamma As String)
			Dim ManufacturerId As Integer = 1 'ex: Emotiv, Inc
			Dim HardwareId As Integer = 1 'ex: Emotive Insight
			Dim ActivityId As Integer = _ActivityId 'ex: Bsaeline, Reading, Writing
			Dim ChannelId As Integer = GetChannelId(channel) 'ex: AF3, AF4, Tz, etc...
			Dim DeviceId As Integer = _DeviceId 'ex: one of the devices bound to a user name
			Dim UserName As String = _UserName
			Dim MeasurementDateTime As Date = Date.Now

			Dim brainActivity = New With {Key ManufacturerId, Key HardwareId, Key ActivityId, Key ChannelId, Key DeviceId, Key UserName, Key MeasurementDateTime, Key theta, Key alpha, Key lowbeta, Key highbeta, Key gamma}
			Dim messageString = JsonConvert.SerializeObject(brainActivity)
			Dim message = New Message(Encoding.ASCII.GetBytes(messageString))
			await deviceClient.SendEventAsync(message) 'Make sure you have internet connection
			WriteLine("{0} > Sending message: {1}", Date.Now, messageString)
		End Sub

		Private Shared Function GetChannelId(ByVal channel As String) As Integer
			Select Case channel
				Case "IED_AF3"
						Return 1
				Case "IED_AF4"
						Return 2
				Case "IED_T7"
						Return 3
				Case "IED_T8"
						Return 4
				Case "IED_O1"
						Return 5
			End Select

			Return 999
		End Function
	End Class
End Namespace
